"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.unzipFile = exports.download = exports.downloadAndUnzip = exports.getTempFilePath = exports.downloadWithCache = void 0;
const follow_redirects_1 = require("follow-redirects");
const fs_1 = require("fs");
const adm_zip_1 = __importDefault(require("adm-zip"));
const crypto_1 = require("crypto");
const userData_1 = require("./userData");
const path_1 = require("path");
async function downloadWithCache(url) {
    const dir = (0, userData_1.getUserDataFolder)(true);
    const cacheDir = (0, path_1.join)(dir, "cache");
    const hash = (0, crypto_1.createHash)("md5").update(url).digest("hex");
    const zipFile = (0, path_1.join)(cacheDir, `${hash}.zip`);
    if ((0, fs_1.existsSync)(zipFile)) {
        console.log(`Get from cache ${zipFile}`);
        return zipFile;
    }
    (0, fs_1.mkdirSync)(cacheDir, { recursive: true });
    await download(url, zipFile);
    return zipFile;
}
exports.downloadWithCache = downloadWithCache;
function getTempFilePath(sufix) {
    const dataDir = (0, userData_1.getUserDataFolder)(true);
    const tmpFolder = (0, path_1.join)(dataDir, "tmp");
    (0, fs_1.mkdirSync)(tmpFolder, { recursive: true });
    const filename = (0, crypto_1.randomUUID)() + sufix;
    return (0, path_1.join)(tmpFolder, filename);
}
exports.getTempFilePath = getTempFilePath;
async function downloadAndUnzip(url, dstDir) {
    const zipFile = await downloadWithCache(url);
    unzipFile(zipFile, dstDir);
}
exports.downloadAndUnzip = downloadAndUnzip;
function download(url, dstFile) {
    const tmpFile = getTempFilePath(".zip");
    console.log(`Fetching ${url} to ${tmpFile}...`);
    return new Promise((resolve, reject) => {
        follow_redirects_1.https.get(url, function (response) {
            const { statusCode } = response;
            if (statusCode !== 200) {
                reject(new Error(`[statusCode ${statusCode}] Network error downloading ${url}`));
            }
            const file = (0, fs_1.createWriteStream)(tmpFile);
            response
                .pipe(file)
                .on("error", () => {
                reject(new Error("Cannot write to file"));
            });
            file.on("finish", function () {
                console.log(`Move temporal file to ${dstFile}`);
                (0, fs_1.renameSync)(tmpFile, dstFile);
                resolve(undefined);
            }).on("error", () => {
                reject(new Error("Cannot write to file"));
            });
        }).on("error", err => {
            reject(err);
        });
    });
}
exports.download = download;
function unzipFile(zipFile, dstDir, skipRootFolder = true) {
    console.log(`Unzipping ${zipFile} to ${dstDir}`);
    const zip = new adm_zip_1.default(zipFile);
    if (skipRootFolder) {
        for (const entry of zip.getEntries()) {
            const relname = entry.entryName.split("/").slice(1).join("/");
            if (entry.isDirectory) {
                const newFolder = (0, path_1.join)(dstDir, relname);
                console.log(`mkdir '${newFolder}'`);
                (0, fs_1.mkdirSync)(newFolder, { recursive: true });
            }
            else {
                const newFolder = (0, path_1.join)(dstDir, (0, path_1.dirname)(relname));
                console.log(`extracting '${entry.entryName}' to '${newFolder}'`);
                zip.extractEntryTo(entry, newFolder, false, true);
            }
        }
    }
    else {
        zip.extractAllTo(dstDir, true, true);
    }
}
exports.unzipFile = unzipFile;
